@@ -798,7 +798,7 @@ static int zip_open(ventry *ve, struct archfile *fil)
     if(ent.method == METHOD_DEFLATE) {
         struct zfile *zfil;
 
-        zfil = av_zfile_new(fil->basefile, fil->nod->offset, ent.crc, 1);
+        zfil = av_zfile_new(fil->basefile, fil->nod->offset, ent.crc, AV_ZFILE_DATA_PLAIN);
         fil->data = zfil;
     }
 
@@ -863,14 +863,15 @@ int av_init_module_uzip(struct vmodule *module)
 {
     int res;
     struct avfs *avfs;
-    struct ext_info zipexts[5];
+    struct ext_info zipexts[6];
     struct archparams *ap;
 
     zipexts[0].from = ".zip",   zipexts[0].to = NULL;
     zipexts[1].from = ".jar",   zipexts[1].to = NULL;
     zipexts[2].from = ".ear",   zipexts[2].to = NULL;
     zipexts[3].from = ".war",   zipexts[3].to = NULL;
-    zipexts[4].from = NULL;
+    zipexts[4].from = ".apk",   zipexts[4].to = NULL;
+    zipexts[5].from = NULL;
 
     res = av_archive_init("uzip", zipexts, AV_VER, module, &avfs);
     if(res < 0)
